/*
 * rotCube.c
 *
 * Date   : November 9, 2020 (Mon)
 * Author : Kenjiro T. MIURA, Shizuoka University, Japan
 * 
 */
#include <stdlib.h>
#include <math.h>
#include "GL/glut.h"
#include <stdio.h>

typedef struct s_vertex {
  float x, y, z; /* W (x, y, z) */
} vertex;

typedef struct s_normal {
  float nx, ny, nz; /* W (x, y, z) */
} normal;

typedef struct s_face {
  int v[4];   /* _CfbNX         */
  int n;      /* @xNgCfbNX */
} face;

typedef struct s_fmodel {
  int faceNumber; /* fɊ܂܂ʂ̐ */
  face *faces;
} fmodel;

/* _̃f[^ */
vertex vertCube[8] =
{ 
   0.0f,  0.0f,  0.0f,
  30.0f,  0.0f,  0.0f,
  30.0f, 15.0f,  0.0f,
  0.0f,  15.0f,  0.0f,
   0.0f,  0.0f, 10.0f,
  30.0f,  0.0f, 10.0f,
  30.0f, 15.0f, 10.0f,
   0.0f, 15.0f, 10.0f  
};

/* ʂ̖@xNgf[^ */
normal normalCube[6] =
{
   0.0f,  0.0f, -1.0f,
   0.0f,  0.0f,  1.0f,
   0.0f, -1.0f,  0.0f,
   1.0f,  0.0f,  0.0f,
   0.0f,  1.0f,  0.0f,
  -1.0f,  0.0f,  0.0f
};

/* ʂ̒_f[^ */
face faceCube[6] =
{
  3, 2, 1, 0,  0,
  4, 5, 6, 7,  1,
  0, 1, 5, 4,  2,
  1, 2, 6, 5,  3,
  2, 3, 7, 6,  4,
  0, 4, 7, 3,  5
};

/* f̃f[^ */
fmodel fCube = {
  6, 
  faceCube
};

/* _ƒ_̃f[^ */
static GLdouble eye[3] = { 80.0, 80.0, 40.0 };					/*_̈ʒu*/
static GLdouble center[3] = { 0.0, 0.0, 0.0 };						/*_*/
static GLdouble up[3]; /* r[AbvxNg */

/* Ɩ̈ʒu */
static GLfloat light_position0[] = { 0.0, 0.0, 1.0, 0.0 };			/*	s@*/
static GLfloat light_position1[] = { 50.0, 0.0, 50.0, 1.0 };		/*@_@*/
static GLfloat light_position2[] = { -50.0, 0.0, 50.0, 1.0 };		/*@_@*/

/* _̉]p */
static int spin_eye = 0;

/* _̐ւ̉] */
void 
rotEyePlus(void)
{
    spin_eye = ( spin_eye + 15 ) % 360;				/*PT*/
}

/* _̕ւ̉] */
void 
rotEyeMinus(void)
{
    spin_eye = ( spin_eye - 15 ) % 360;				/*PT*/
}
static int projection=20;

/* Ɩ̐ݒ */
void 
initLights(void)
{
    GLfloat light0_diffuse[] = { 0.9, 0.9, 0.9, 1.0 }; /* gU */
    GLfloat light1_diffuse[] = { 0.5, 0.5, 0.5, 1.0 }; /* gU */

    GLfloat light_specular[] = { 0.3, 0.3, 0.3, 1.0 }; /* ʐ */

    GLfloat lmodel_ambient[] = { 0.2, 0.2, 0.2, 1.0 }; /*  */

    glLightModelfv(GL_LIGHT_MODEL_AMBIENT, lmodel_ambient );	/**/

    glLightfv ( GL_LIGHT0, GL_DIFFUSE, light0_diffuse );		/*light0̊gU̐ݒ*/
    //glLightfv ( GL_LIGHT0, GL_SPECULAR, light_specular );

    glLightfv ( GL_LIGHT1, GL_DIFFUSE, light1_diffuse );		/*light1̊gU̐ݒ*/
    glLightfv ( GL_LIGHT1, GL_SPECULAR, light_specular );		/*light1̋ʔˌ̐ݒ*/

    glLightfv ( GL_LIGHT2, GL_DIFFUSE, light1_diffuse );		/*light2̊gU̐ݒ*/
    glLightfv ( GL_LIGHT2, GL_SPECULAR, light_specular );		/*light2̋ʔˌ̐ݒ*/

    glEnable ( GL_LIGHTING );									/*ƖʂLɂ*/
    glEnable ( GL_LIGHT0 );										/*light0ɂƖLɂ*/
    glEnable ( GL_LIGHT1 );										/*light1ɂƖLɂ*/
    glEnable ( GL_LIGHT2 );										/*light2ɂƖLɂ*/
}

/*IuWFNg̕`*/
void 
drawFModel(fmodel *fmo) {
  int i, j;         /* JE^     */
  float x, y, z;    /* _̍Wl */
  float nx, ny, nz; /* @xNg̍Wl */
  int faceNum;

  faceNum = fmo->faceNumber;									/*\̃|C^(fmo)̎w\̂̃o[(faceNumber)ւ̃ANZX*/
  glBegin(GL_QUADS);											/*lp``*/
    for (i=0; i<faceNum; ++i) {
		nx = normalCube[fmo->faces[i].n].nx;					
        ny = normalCube[fmo->faces[i].n].ny;
        nz = normalCube[fmo->faces[i].n].nz;
        glNormal3f(nx, ny, nz);									/*P̖ʂ\S_̖@̍W*/
		for (j=0; j<4; ++j) {									
		    x = vertCube[fmo->faces[i].v[j]].x;
		    y = vertCube[fmo->faces[i].v[j]].y;
		    z = vertCube[fmo->faces[i].v[j]].z;
		    glVertex3f(x, y, z);								/*lp`\S_i_j̍W*/
		}
    }
	glEnd();
}

/* W̕` */
void 
drawAxis() {
  glDisable( GL_LIGHTING );					/* Ɩʂ𖳌ɂB */

  /* x(bh) */
  glColor3f(1.0f, 0.0f, 0.0f);
  glBegin(GL_LINES);
    glVertex3f(-100.0f, 0.0f, 0.0f);
    glVertex3f(100.0f, 0.0f, 0.0f);
  glEnd();

  /* y(O[) */
  glColor3f(0.0f, 1.0f, 0.0f);
  glBegin(GL_LINES);
    glVertex3f(0.0f, -100.0f, 0.0f);
    glVertex3f(0.0f, 100.0f, 0.0f);
  glEnd();

  /* z(u[) */
  glColor3f(0.0f, 0.0f, 1.0f);
  glBegin(GL_LINES);
    glVertex3f(0.0f, 0.0f, -100.0f);
    glVertex3f(0.0f, 0.0f, 100.0f);
  glEnd();

  glEnable(GL_LIGHTING);					/* ƖʂLɂB */
}

void
ourDisplay(void)
{
  GLfloat material_color[4] = { 1.0, 0.0, 0.0, 1.0 }; /* gUː */
  GLfloat material_specular[4] = { 0.2, 0.2, 0.2, 1.0 }; /* ʔː */

  /* obt@̃NA */
  glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

  /* ʔː̃Zbg */
  glMaterialfv(GL_FRONT, GL_SPECULAR, material_specular);	/*ގ̓iOʁAʔˌA(material_specular)j̐ݒ*/

  /* fr[s */
  glMatrixMode ( GL_MODELVIEW );
  glLoadIdentity ();
  /* r[sZbgB*/    
  gluLookAt(eye[0], eye[1], eye[2], center[0], center[1], center[2],
            up[0], up[1], up[2]);									/*_̐ݒi_A_Aj*/

  glLightfv ( GL_LIGHT0, GL_POSITION, light_position0 );			/*light0light_position0̈ʒuɐݒu*/
  glLightfv ( GL_LIGHT1, GL_POSITION, light_position1 );			/*light0light_position1̈ʒuɐݒu*/
  glLightfv ( GL_LIGHT1, GL_POSITION, light_position2 );			/*light0light_position2̈ʒuɐݒu*/

  /* _̉]ړ	 */
  glRotated ( (GLdouble) spin_eye, 0.0, 0.0, 1.0 );					/*z̕spin_eyex]*/

  /* x, y, z̕` */
  drawAxis();

  /* bh */
  glMaterialfv(GL_FRONT, GL_DIFFUSE, material_color);		/*ގ̓iOʁAgUAimaterial_color0jj̐ݒ*/			

  drawFModel( &fCube );										/*fCubeɂdrawModels*/
  
  glFlush();
}

void 
ourInit (void) 
{
  glClearColor(0.0, 0.0, 0.0, 1.0);		/*wiF̎w*/
  glDepthFunc ( GL_LESS );				/*fvXeXĝ߂̔r֐GL_LESSiÕtOg`j*/
  glEnable ( GL_DEPTH_TEST );			/*fvXeXgLɂ*/

  initLights();							/*Ɩ̐ݒ*/
}

/*
 *  EChEŏɃI[vEChEړ⃊TCYꂽ
 *  Ă΂B
 */
void 
ourReshape(int width, int height)
{
    int i; /* JE^[ */
    GLdouble vector[3]; /* r[AbvxNgvZpxNg */
    GLdouble norm; /* xNg̃m */

    glViewport (0, 0, width, height);
    glMatrixMode (GL_PROJECTION);
    glLoadIdentity();
	gluPerspective(60.0, (GLfloat) width/(GLfloat) height, 10.0, 1000.0);
	glMatrixMode(GL_MODELVIEW);
    glLoadIdentity();

    /* r[AbvxNgvZB*/
    for ( i=0; i<3; i++ ){
        vector[i] = center[i] - eye[i];
    }
    up[0] = - vector[0] * vector[2];
    up[1] = - vector[1] * vector[2];
    up[2] = vector[0] * vector[0] + vector[1] * vector[1];
    norm = up[0] * up[0] + up[1] * up[1] + up[2] * up[2];
    norm = sqrt ( norm );

    for ( i=0; i<3; ++i ) up[i] /= norm;
}


void ourKeyboard(unsigned char key,int x,int y)
{
	switch(key)
	{
	case 'p':
	case 'P':
		rotEyePlus();
		break;

	case 'm':
	case 'M':
		rotEyeMinus();
	}
		
	glutPostRedisplay();/* ourKeyboard()Ă΂ꂽourDisplay()ĂԂƂw@*/
}

/*  C */
int main(int argc, char **argv) 
{
	glutInit(&argc, argv);
	glutInitDisplayMode(GLUT_SINGLE | GLUT_RGB);
	glutInitWindowSize(400, 400);
	glutInitWindowPosition(100, 100);
	glutCreateWindow("exer6");
	ourInit();
	glutReshapeFunc(ourReshape);
	glutDisplayFunc(ourDisplay);
	glutKeyboardFunc(ourKeyboard);

	glutMainLoop();
	return 0;
}



